// Copyright (c) 2006,2007 Polybus Systems Corp., Westford, MA, USA 
// All rights reserved 

// Redistribution and use in source and binary forms, with or without 
// modification, are permitted provided that the following conditions 
// are met: 

// 1. Redistributions of source code must retain the above copyright 
// notice, this list of conditions and the following disclaimer. 

// 2. Redistributions in binary form must reproduce the above copyright 
// notice, this list of conditions and the following disclaimer in the 
// documentation and/or other materials provided with the distribution. 

// 3. All advertising materials mentioning features or use of this software 
// must display the following acknowledgement: 
// This product includes software developed by the Polybus Systems Corporation. 

// 4. Neither the name of Polybus System Corporation or the names of its contributors 
// may be used to endorse or promote products derived from this software 
// without specific prior written permission. 

// THIS SOFTWARE IS PROVIDED BY POLYBUS SYSTEMS CORP AND IT'S CONTRIBUTORS "AS IS" AND 
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
// ARE DISCLAIMED. IN NO EVENT SHALL THE POLYBUS SYSTEMS CORP OR CONTRIBUTORS BE LIABLE 
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
// SUCH DAMAGE. 


#include "sys_basher.h"
#include <form.h>


void cursesConfig(config_args *args)
{
  SInt32 myChar;
  SInt32 x,y,width,height;
  WINDOW *myWin=0;

  initscr(); /* Start curses mode */
  raw(); /* Line buffering disabled */
  cbreak();
  keypad(stdscr, TRUE); /* We get F1, F2 etc.. */
  noecho(); /* Don't echo() while we do getch */
  
  height = 10;
  width = 20;
  y = (LINES - height) / 2; /* Calculating for a center placement */
  x = (COLS - width) / 2; /* of the window */
  printw("Press F1 to exit");
  refresh();
  myWin = create_newwin(height, width, y, x);
  myChar = getch();
  while((myChar != '\n') && (myChar != 'q'))
    { 
      switch(myChar)
	{ 
	case KEY_LEFT:
	  destroy_win(myWin);
	  x--;
	  if(x < 0) x = 0;
	  myWin = create_newwin(height, width, y,x);
	  break;
	case KEY_RIGHT:
	  destroy_win(myWin);
	  x++;
	  if(x >= COLS) x = COLS -1;
	  myWin = create_newwin(height, width, y,++x);
	  break;
	case KEY_UP:
	  destroy_win(myWin);
	  y--;
	  if(y< 0) y = 0;
	  myWin = create_newwin(height, width, y,x);
	  break;
	case KEY_DOWN:
	  destroy_win(myWin);
	  y++;
	  if(y >= LINES) y = LINES - 1;
	  myWin = create_newwin(height, width, y,x);
	  break;
	}
      myChar = getch();
    }
  printw("Run/Quit (r/q):"); 
  refresh(); /* Print it on to the real screen */
  myChar = getch(); /* Wait for user input */
  endwin(); /* End curses mode */
  if((myChar == 'r') || (myChar == 'R')) return;
  exit(0);
}

WINDOW *create_newwin(int height, int width, int starty, int startx)
{ 
  WINDOW *local_win;
  local_win = newwin(height, width, starty, startx);
  box(local_win, 0 , 0); /* 0, 0 gives default characters
			  * for the vertical and horizontal
			  * lines */
  wrefresh(local_win); /* Show that box */
  return (local_win);
}

void destroy_win(WINDOW *local_win)
{
  /* box(local_win, ' ', ' '); : This won't produce the desired
   * result of erasing the window. It will leave it's four corners
   * and so an ugly remnant of window.
   */
  wborder(local_win, ' ', ' ', ' ',' ',' ',' ',' ',' ');
  /* The parameters taken are
   * 1. win: the window on which to operate
   * 2. ls: character to be used for the left side of the window
   * 3. rs: character to be used for the right side of the window
   * 4. ts: character to be used for the top side of the window
   * 5. bs: character to be used for the bottom side of the window
   * 6. tl: character to be used for the top left corner of the window
   * 7. tr: character to be used for the top right corner of the window
   * 8. bl: character to be used for the bottom left corner of the window
   * 9. br: character to be used for the bottom right corner of the window
   */
  wrefresh(local_win);
  delwin(local_win);
}

void sysForms(config_args *args)
{
  FIELD *field[3];
  FORM *my_form;
  WINDOW *my_form_win;
  int ch, rows, cols;
  /* Initialize curses */
  initscr();
  start_color();
  cbreak();
  noecho();
  keypad(stdscr, TRUE);
  /* Initialize few color pairs */
  init_pair(1, COLOR_BLACK, COLOR_WHITE);
  /* Initialize the fields */
  field[0] = new_field(1, 10, 6, 1, 0, 0);
  field[1] = new_field(1, 10, 8, 1, 0, 0);
  field[2] = NULL;
  /* Set field options */
  set_field_back(field[0], A_UNDERLINE);
  field_opts_off(field[0], O_AUTOSKIP); /* Don't go to next field when this */
  /* Field is filled up */
  set_field_back(field[1], A_UNDERLINE);
  field_opts_off(field[1], O_AUTOSKIP);
  /* Create the form and post it */
  my_form = new_form(field);

  /* Calculate the area required for the form */
  scale_form(my_form, &rows, &cols);
  /* Create the window to be associated with the form */
  my_form_win = newwin(rows + 4, cols + 4, 4, 4);
  wattron(my_form_win,COLOR_PAIR(1));
  keypad(my_form_win, TRUE);
  /* Set main window and sub window */
  set_form_win(my_form, my_form_win);
  set_form_sub(my_form, derwin(my_form_win, rows, cols, 2, 2));
  /* Print a border around the main window and print a title */
  box(my_form_win, 0, 0);
  print_in_middle(my_form_win, 1, 0, cols + 4, "My Form", COLOR_PAIR(1));
  post_form(my_form);
  wrefresh(my_form_win);
  mvprintw(LINES - 2, 0, "Use UP, DOWN arrow keys to switch between fields");
  refresh();
  /* Loop through to get user requests */
  while((ch = wgetch(my_form_win)) != KEY_F(1))
    { switch(ch)
	{ case KEY_DOWN:
	    /* Go to next field */
	    form_driver(my_form, REQ_NEXT_FIELD);
	    /* Go to the end of the present buffer */
	    /* Leaves nicely at the last character */
	    form_driver(my_form, REQ_END_LINE);
	    break;
	case KEY_UP:
	  /* Go to previous field */
	  form_driver(my_form, REQ_PREV_FIELD);
	  form_driver(my_form, REQ_END_LINE);
	  break;
	default:
	  /* If this is a normal character, it gets */
	  /* Printed */
	  form_driver(my_form, ch);
	  break;
	}
    }
  /* Un post form and free the memory */
  unpost_form(my_form);
  free_form(my_form);
  free_field(field[0]);
  free_field(field[1]);
  endwin();
  return;
}
void print_in_middle(WINDOW *win, int starty, int startx, int width, char *string, chtype color)
{ int length, x, y;
  float temp;
  if(win == NULL)
    win = stdscr;
  getyx(win, y, x);
  if(startx != 0)
    x = startx;
  if(starty != 0)
    y = starty;
  if(width == 0)
    width = 80;
  length = strlen(string);
  temp = (width - length)/ 2;
  x = startx + (int)temp;
  wattron(win, color);
  mvwprintw(win, y, x, "%s", string);
  wattroff(win, color);
  refresh();
}
